use crossterm::event::KeyCode;
use readmap::types::Section;
use std::path::PathBuf;
use tool_helper::Error;
use tui::{
    layout::{Alignment, Constraint, Direction, Layout, Rect},
    style::{Color, Style},
    widgets::{Block, Borders, BorderType, Paragraph}
};

pub type EventReceiver = std::sync::mpsc::Receiver<Event<crossterm::event::KeyEvent>>;
pub type Terminal      = tui::Terminal<tui::backend::CrosstermBackend<std::io::Stdout>>;

pub enum Event<I> {
    Input(I),
    Tick,
}

pub enum UIState {
    Alive,
    Terminated
}

pub struct ConsoleUI {
    rx:         EventReceiver,
    terminal:   Terminal,
    frame:      Rect,
    sub_frames: Vec<Rect>
}

impl ConsoleUI {
    pub fn new(rx: EventReceiver, terminal: Terminal) -> ConsoleUI {
        ConsoleUI{rx, terminal, frame: Rect::default(), sub_frames: Vec::default()}
    }

    pub fn update(&self) -> Result<UIState, Error> {
        match self.rx.recv()? {
            Event::Input(event) => {
                match event.code {
                    KeyCode::Char('q') => Ok(UIState::Terminated),
                    _                  => Ok(UIState::Alive)
                }
            },
            Event::Tick         => Ok(UIState::Alive)
        }
    }

    pub fn render(&mut self) -> Result<(), Error> {
        self.terminal.draw(|frame| {
            Self::update_sub_frames(&mut self.sub_frames, &mut self.frame, frame.size());

            frame.render_widget(Self::create_titel(), self.sub_frames[0]);
            frame.render_widget(Self::create_titel(), self.sub_frames[1]);
            frame.render_widget(Self::create_titel(), self.sub_frames[2]);
        })?;

        Ok(())
    }

    pub fn close(mut self) -> Result<(), Error> {
        crossterm::terminal::disable_raw_mode()?;
        self.terminal.show_cursor()?;
        self.terminal.clear()?;

        Ok(())
    }

    fn update_sub_frames(sub_frames: &mut Vec<Rect>, frame: &mut Rect, new_frame: Rect) {
        if new_frame  != *frame {
            *sub_frames = Layout::default().direction(Direction::Vertical).constraints([
                Constraint::Length(3),
                Constraint::Length(3),
                Constraint::Min(3)
            ]).split(new_frame);
            *frame = new_frame;
        }
    }

    fn create_titel<'a>() -> Paragraph<'a> {
        Paragraph::new("psxreadmap")
            .style(Style::default().fg(Color::White))
            .alignment(Alignment::Center)
            .block(Block::default()
                .borders(Borders::ALL)
                .style(Style::default().fg(Color::White))
                .border_type(BorderType::Plain)
            )
    }
}

pub fn load_memory_map(file_path: Option<PathBuf>) -> Result<Vec<Section>, Error> {
    readmap::scan(tool_helper::open_input(file_path)?)
}